import React, { useContext } from 'react'
import PropTypes from 'prop-types'
import { GraphContext, DragContext } from '../utils/index.js'
import * as d3 from 'd3-selection'
import { drag as d3Drag } from 'd3-drag'


const Draggable = ({
	children,
	onDragStart,
	onDragEnd
}) => {
	const { simulation } = useContext(GraphContext)
	const drag = d3Drag()

	drag.on('start', d => {
		if (onDragStart) onDragStart(d3.event)
		if (!d3.event.active) simulation.alphaTarget(0.3).restart()
		d.fx = d.x
		d.fy = d.y
	})

	drag.on('drag', d => {
		d.fx = d3.event.x
		d.fy = d3.event.y
	})

	drag.on('end', d => {
		if(onDragEnd) onDragEnd(d3.event)
		if (!d3.event.active) simulation.alphaTarget(0)
		d.fx = null
		d.fy = null
	})

	return (
		<DragContext.Provider value={drag}>
			{children}
		</DragContext.Provider>
	)
}

Draggable.propTypes = {
	children: PropTypes.node
}

export default Draggable