import React, { useEffect, useRef } from 'react'
import PropTypes from 'prop-types'
import { GraphContext } from '../utils/index.js'
import * as d3 from 'd3-force'


const Graph = ({
	width = 500,
	height = 500,
	nodes = [],
	links = [],
	children,
	forces = {
		collision: d3.forceCollide().radius(30).strength(0.25),
		charge: d3.forceManyBody().strength(-1000).theta(0.25),
		x: d3.forceX(),
		y: d3.forceY()
	}
}) => {
	const svg = useRef(null)
	const zoomGroup = useRef(null)
	const linksGroup = useRef(null)
	const nodesGroup = useRef(null)
	const brushGroup = useRef(null)
	const simulation = d3.forceSimulation(nodes)

	simulation.force('link', d3.forceLink(links))

	Object.keys(forces).forEach(key => {
		simulation.force(key, forces[key])
	})

	return (
		<GraphContext.Provider value={{
			simulation,
			links,
			nodes,
			svg,
			zoomGroup,
			linksGroup,
			nodesGroup,
			brushGroup
		}}>
			<svg
				ref={svg}
				width={width}
				height={height}
				viewBox={`${-width / 2} ${-height / 2} ${width} ${height}`}
			>
				<g ref={zoomGroup}>
					<g ref={linksGroup} />
					<g ref={nodesGroup} />
					<g ref={brushGroup} />
				</g>
				{children}
			</svg>
		</GraphContext.Provider>
	)
}

Graph.propTypes = {
	width: PropTypes.number,
	height: PropTypes.number
}

export default Graph