import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext } from '../utils/index.js'
import * as d3 from 'd3-selection'


const Links = ({
	attributes = {
		'stroke': '#ccc',
		'stroke-width': 1.5
	},
	style = {},
	onTick = link => {
		link.attr('x1', d => d.source.x)
		link.attr('y1', d => d.source.y)
		link.attr('x2', d => d.target.x)
		link.attr('y2', d => d.target.y)
	}
}) => {
	const {simulation, links, linksGroup} = useContext(GraphContext)

	useEffect(() => {
		const group = d3.select(linksGroup.current)
		const link = group.selectAll('line').data(links).join('line')
		const attrs = Object.keys({ ...attributes })
		const styles = Object.keys(style)

		attrs.forEach(key => link.attr(key, attributes[key]))
		styles.forEach(key => link.style(key, style[key]))

		simulation.on('tick.links', () => onTick(link))
	})

	return null
}

Links.propTypes = {
	attributes: PropTypes.object,
	style: PropTypes.object,
	onTick: PropTypes.func
}

export default Links