import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext, DragContext, SelectContext } from '../utils/index.js'
import { selectNode, deselectNodes, selectNodes } from '../utils/selectable.js'
import { savePreviousSelection } from '../utils/selectable.js'
import * as d3 from 'd3-selection'


const Nodes = ({
	element = 'circle',
	attributes = { r: 20 },
	style = {},
	children = null,
	onRender = () => {},
	onMouseOver,
	onMouseOut,
	onClick,
	onTick = node => {
		node.attr('cx', d => d.x)
		node.attr('cy', d => d.y)
	}
}) => {
	const {svg, simulation, nodes, nodesGroup} = useContext(GraphContext)
	const {brush} = useContext(SelectContext)
	const drag = useContext(DragContext)

	useEffect(() => {
		const container = d3.select(svg.current)
		const group = d3.select(nodesGroup.current)
		const node = group.selectAll(element).data(nodes).join(element)
		const attrs = Object.keys({ ...attributes })
		const styles = Object.keys(style)

		attrs.forEach(key => node.attr(key, attributes[key]))
		styles.forEach(key => node.style(key, style[key]))

		node.each(onRender)
		node.on('mouseover', onMouseOver)
		node.on('mouseout', onMouseOut)
		node.on('click', onClick)
		simulation.on('tick.nodes', () => onTick(node))

		if (drag) node.call(drag)
		if (brush) {
			brush.on('brush', () => selectNodes(node))
			brush.on('start', () => savePreviousSelection(node))
			container.on('click', () => deselectNodes(node))
			node.on('click', selectNode)
		}
	})

	return children
}

Nodes.propTypes = {
	element: PropTypes.string,
	attributes: PropTypes.object,
	style: PropTypes.object,
	onMouseOver: PropTypes.func,
	onMouseOut: PropTypes.func,
	onClick: PropTypes.func,
	onTick: PropTypes.func
}

export default Nodes