import React, { useContext, useEffect } from 'react'
import PropTypes from 'prop-types'
import { GraphContext, SelectContext } from '../utils/index.js'
import { brush as d3Brush } from 'd3-brush'
import * as d3 from 'd3-selection'


const Selectable = ({
	children = null
}) => {
	const {svg, brushGroup} = useContext(GraphContext)
	const brush = d3Brush()

	useEffect(() => {
		const { clientWidth, clientHeight } = svg.current
		const container = d3.select(svg.current)
		const body = d3.select('body')
		const group = d3.select(brushGroup.current)
		const viewbox = container.attr('viewBox').split(' ').map(str => Number(str))
		const activate = event => {
			const overlay = group.select('.overlay')
			const shifted = event && event.shiftKey

			overlay.style('cursor', shifted ? 'crosshair' : 'inherit')
			overlay.style('pointer-events', shifted ? 'all' : 'none')
		}
		
		brush.extent([[viewbox[0], viewbox[1]], [viewbox[2], viewbox[3]]])
		brush.filter(() => d3.event.shiftKey)
		brush.keyModifiers(false)
		brush.on('end', () => { try { brush.clear(group) } catch (err) {} })

		group.call(brush)
		activate()
		body.on('keydown', () => activate(d3.event))
		body.on('keyup', activate)
	})

	return (
		<SelectContext.Provider value={{ brush }}>
			{children}
		</SelectContext.Provider>
	)
}

Selectable.propTypes = {
	children: PropTypes.node
}

export default Selectable