import React, { useContext, useState } from 'react'
import PropTypes from 'prop-types'
import { GraphContext } from '../utils/index.js'
import * as d3 from 'd3-selection'


const Tooltip = ({
	component: Component,
	children
}) => {
	const {svg} = useContext(GraphContext)
	const [props, setProps] = useState(null)

	return (
		<React.Fragment>
			{children({
				show: (d, i, elements) => {
					d3.select(elements[i]).on('mousemove.tooltip', () => {
						const {x, y} = svg.current.getBoundingClientRect()

						setProps({
							data: d,
							element: elements[i],
							position: {
								x: d3.event.clientX - x,
								y: d3.event.clientY - y
							}
						})
					})
				},
				hide: () => {
					if (props) {
						d3.select(props.element).on('mousemove.tooltip', null)
						setProps(null)
					}
				}
			})}
			{props && (
				<foreignObject
					id="tooltip"
					width="100%"
					height="100%"
					x="-50%"
					y="-50%"
					style={{
						pointerEvents: 'none'
					}}
				>
					<div
						style={{
							position: 'fixed',
							top: props.position.y,
							left: props.position.x,
							pointerEvents: 'all',
							cursor: 'auto'
						}}
					>
						<Component {...props} />
					</div>
				</foreignObject>
			)}
		</React.Fragment>
	)
}

Tooltip.propTypes = {
	component: PropTypes.func.isRequired,
	children: PropTypes.func.isRequired
}

export default Tooltip